"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferSols = exports.transferSpl = exports.SolanaSwapClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const RaydiumSwap_1 = __importDefault(require("../server/RaydiumSwap"));
const utils_1 = require("../utils");
class SolanaSwapClient {
    constructor(poolKeys, devnet = false) {
        this.rpc = 'https://api.mainnet-beta.solana.com';
        this.rpc = devnet ? 'https://api.devnet.solana.com' : 'https://api.mainnet-beta.solana.com';
        this.connection = new web3_js_1.Connection(this.rpc);
        if (!poolKeys || poolKeys.length === 0) {
            throw new Error('Pool keys not loaded');
        }
        this.poolKeys = poolKeys;
        if (!this.poolKeys || this.poolKeys.length === 0) {
            throw new Error('Pool keys not loaded');
        }
    }
    swap(fromPrivKeyBs58_1, tokenIn_1, tokenOut_1, amountIn_1) {
        return __awaiter(this, arguments, void 0, function* (fromPrivKeyBs58, tokenIn, tokenOut, amountIn, slippage = "5", // 5%
        priceImpact = null, maxRetries = 20) {
            const owner = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
            const amountInSols = (0, utils_1.convertNumber)(amountIn);
            const raydiumSwap = new RaydiumSwap_1.default(this.rpc, owner, this.poolKeys);
            const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenIn, tokenOut);
            if (!poolInfo) {
                throw new Error("Pool info not found");
            }
            const transaction = yield raydiumSwap.getSwapTransaction(tokenOut, amountInSols, poolInfo, slippage, priceImpact);
            const txid = yield raydiumSwap.sendLegacyTransaction(transaction, maxRetries);
            const latestBlockhash = yield this.connection.getLatestBlockhash();
            yield this.connection.confirmTransaction({
                signature: txid,
                blockhash: latestBlockhash.blockhash,
                lastValidBlockHeight: latestBlockhash.lastValidBlockHeight,
            }, "confirmed");
            const tx = yield this.connection.getTransaction(txid, { "commitment": "confirmed", "maxSupportedTransactionVersion": 1 });
            const preTokenOutBalance = tx.meta.preTokenBalances.find(obj => obj.mint === tokenOut && obj.owner === owner.publicKey.toBase58());
            const postTokenOutBalance = tx.meta.postTokenBalances.find(obj => obj.mint === tokenOut && obj.owner === owner.publicKey.toBase58());
            const amountOut = postTokenOutBalance.uiTokenAmount.uiAmount - preTokenOutBalance.uiTokenAmount.uiAmount;
            return {
                txid,
                amountOut
            };
        });
    }
    calcAmountOut(fromPrivKeyBs58_1, tokenIn_1, tokenOut_1, amountIn_1) {
        return __awaiter(this, arguments, void 0, function* (fromPrivKeyBs58, tokenIn, tokenOut, amountIn, slippage = "5", // 5%
        priceImpact = null) {
            const owner = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
            const amountInSols = (0, utils_1.convertNumber)(amountIn);
            const raydiumSwap = new RaydiumSwap_1.default(this.rpc, owner, this.poolKeys);
            const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenIn, tokenOut);
            if (!poolInfo) {
                throw new Error("Pool info not found");
            }
            const directionIn = poolInfo.quoteMint.toString() == tokenOut;
            const amountOut = yield raydiumSwap.calcAmountOut(poolInfo, amountInSols, directionIn, slippage, priceImpact);
            const amountOutParsed = amountOut.amountOut.raw.toNumber() /
                10 ** amountOut.amountOut.currency.decimals;
            return {
                amountIn: amountInSols,
                amountOut: amountOutParsed,
            };
        });
    }
}
exports.SolanaSwapClient = SolanaSwapClient;
function transferSpl(connection, fromPrivKeyBs58, token, amount, receiver) {
    return __awaiter(this, void 0, void 0, function* () {
        const to = (0, utils_1.convertPubkey)(receiver);
        const from = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
        const amountInSols = (0, utils_1.convertNumber)(amount);
        const fromAssociatedTokenAccount = yield (0, spl_token_1.getAssociatedTokenAddress)((0, utils_1.convertPubkey)(token), from.publicKey);
        const toAssociatedTokenAccount = yield (0, spl_token_1.getAssociatedTokenAddress)((0, utils_1.convertPubkey)(token), to);
        const tx = new web3_js_1.Transaction();
        try {
            const toAccountInfo = yield connection.getAccountInfo(toAssociatedTokenAccount);
            if (!toAccountInfo) {
                tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(from.publicKey, toAssociatedTokenAccount, to, (0, utils_1.convertPubkey)(token)));
            }
        }
        catch (e) {
            tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(from.publicKey, toAssociatedTokenAccount, to, (0, utils_1.convertPubkey)(token)));
        }
        tx.add((0, spl_token_1.createTransferInstruction)(fromAssociatedTokenAccount, toAssociatedTokenAccount, from.publicKey, amountInSols * web3_js_1.LAMPORTS_PER_SOL));
        tx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
        tx.feePayer = from.publicKey;
        const recoverTx = web3_js_1.Transaction.from(tx.serialize({ requireAllSignatures: false }));
        // transaction sign
        recoverTx.sign(from);
        const signature = yield connection.sendRawTransaction(recoverTx.serialize());
        return signature;
    });
}
exports.transferSpl = transferSpl;
function transferSols(connection, fromPrivKeyBs58, amount, receiver) {
    return __awaiter(this, void 0, void 0, function* () {
        const to = (0, utils_1.convertPubkey)(receiver);
        const from = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
        const amountInSols = (0, utils_1.convertNumber)(amount);
        const transferInstruction = [
            web3_js_1.SystemProgram.transfer({
                fromPubkey: from.publicKey,
                toPubkey: to,
                lamports: amountInSols * web3_js_1.LAMPORTS_PER_SOL
            }),
        ];
        let { blockhash } = yield connection.getLatestBlockhash();
        const message = new web3_js_1.TransactionMessage({
            payerKey: from.publicKey,
            recentBlockhash: blockhash,
            instructions: transferInstruction,
        }).compileToV0Message();
        const transaction = new web3_js_1.VersionedTransaction(message);
        transaction.sign([from]);
        const signature = yield connection.sendRawTransaction(transaction.serialize());
        return signature;
    });
}
exports.transferSols = transferSols;
